import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';

class ExpandableChip extends StatefulWidget {
  final IconData icon;
  final Widget child;
  final Function() onTap;
  final double shrinkSize, expandedSize;
  final Color? iconColor, backgroundColor;
  final Duration duration, showChildDuration;
  final EdgeInsets? margin, padding;
  final Curve curve;
  final Matrix4? transform;
  final BoxBorder? border;
  final List<BoxShadow>? boxShadow;
  final Gradient? gradient;
  final TextDirection? direction;

  const ExpandableChip({
    Key? key,
    required this.icon,
    required this.onTap,
    required this.shrinkSize,
    required this.expandedSize,
    required this.child,
    this.duration = const Duration(milliseconds: 200),
    this.iconColor,
    this.backgroundColor,
    this.margin,
    this.padding,
    this.curve = Curves.linear,
    this.transform,
    this.border,
    this.boxShadow,
    this.gradient,
    this.showChildDuration = const Duration(milliseconds: 200),
    this.direction,
  }) : super(key: key);

  @override
  _ExpandableChipState createState() => _ExpandableChipState();
}

class _ExpandableChipState extends State<ExpandableChip> {
  Stopwatch _stopWatch = Stopwatch();
  bool _widgetVisibility = false;
  double? _containerWidth;

  TextDirection get _direction => widget.direction ?? Directionality.of(context);

  @override
  void initState() {
    _containerWidth = widget.shrinkSize;

    super.initState();
  }

  @override
  void dispose() {
    _stopWatch.stop();
    super.dispose();
  }

  void changeState({bool shouldExpand = false}) {
    if (shouldExpand) {
      _stopWatch.start();
      Timer(widget.showChildDuration, () {
        if (_stopWatch.isRunning) {
          _widgetVisibility = true;
          setState(() {});
        }
      });
      _containerWidth = widget.expandedSize;
    } else {
      _stopWatch.stop();
      _containerWidth = widget.shrinkSize;
      _widgetVisibility = false;
    }
    setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    final List<Widget> chipChildren = <Widget>[
      Center(
        child: Icon(
          widget.icon,
          size: widget.shrinkSize / 2,
          color: widget.iconColor,
        ),
      ),
      if(!_widgetVisibility && _containerWidth == widget.expandedSize) SizedBox(width: 1),
      if (_widgetVisibility) widget.child,
    ];

    return GestureDetector(
      onTap: widget.onTap,
      child: MouseRegion(
        cursor: SystemMouseCursors.click,
        onEnter: (p) {
          changeState(shouldExpand: true);
        },
        onExit: (p) {
          changeState();
        },
        child: AnimatedContainer(
          margin: widget.margin,
          padding: widget.padding,
          curve: widget.curve,
          transform: widget.transform,
          height: widget.shrinkSize,
          width: _containerWidth,
          duration: widget.duration,
          decoration: BoxDecoration(
            border: widget.border,
            boxShadow: widget.boxShadow,
            gradient: widget.gradient,
            color: widget.backgroundColor,
            borderRadius: BorderRadius.circular(widget.shrinkSize / 2),
          ),
          child: Row(
            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
            mainAxisSize: MainAxisSize.min,
            children: _direction == TextDirection.ltr ? chipChildren : chipChildren.reversed.toList(),
          ),
        ),
      ),
    );
  }
}
